/**
 * Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
 * SPDX-License-Identifier: Apache-2.0.
 */

#pragma once
#include <aws/core/utils/UUID.h>
#include <aws/core/utils/memory/stl/AWSString.h>
#include <aws/core/utils/memory/stl/AWSVector.h>
#include <aws/ec2/EC2Request.h>
#include <aws/ec2/EC2_EXPORTS.h>
#include <aws/ec2/model/TagSpecification.h>

#include <utility>

namespace Aws {
namespace EC2 {
namespace Model {

/**
 */
class CreateTrafficMirrorSessionRequest : public EC2Request {
 public:
  AWS_EC2_API CreateTrafficMirrorSessionRequest() = default;

  // Service request name is the Operation name which will send this request out,
  // each operation should has unique request name, so that we can get operation's name from this request.
  // Note: this is not true for response, multiple operations may have the same response name,
  // so we can not get operation's name from response.
  inline virtual const char* GetServiceRequestName() const override { return "CreateTrafficMirrorSession"; }

  AWS_EC2_API Aws::String SerializePayload() const override;

 protected:
  AWS_EC2_API void DumpBodyToUrl(Aws::Http::URI& uri) const override;

 public:
  ///@{
  /**
   * <p>The ID of the source network interface.</p>
   */
  inline const Aws::String& GetNetworkInterfaceId() const { return m_networkInterfaceId; }
  inline bool NetworkInterfaceIdHasBeenSet() const { return m_networkInterfaceIdHasBeenSet; }
  template <typename NetworkInterfaceIdT = Aws::String>
  void SetNetworkInterfaceId(NetworkInterfaceIdT&& value) {
    m_networkInterfaceIdHasBeenSet = true;
    m_networkInterfaceId = std::forward<NetworkInterfaceIdT>(value);
  }
  template <typename NetworkInterfaceIdT = Aws::String>
  CreateTrafficMirrorSessionRequest& WithNetworkInterfaceId(NetworkInterfaceIdT&& value) {
    SetNetworkInterfaceId(std::forward<NetworkInterfaceIdT>(value));
    return *this;
  }
  ///@}

  ///@{
  /**
   * <p>The ID of the Traffic Mirror target.</p>
   */
  inline const Aws::String& GetTrafficMirrorTargetId() const { return m_trafficMirrorTargetId; }
  inline bool TrafficMirrorTargetIdHasBeenSet() const { return m_trafficMirrorTargetIdHasBeenSet; }
  template <typename TrafficMirrorTargetIdT = Aws::String>
  void SetTrafficMirrorTargetId(TrafficMirrorTargetIdT&& value) {
    m_trafficMirrorTargetIdHasBeenSet = true;
    m_trafficMirrorTargetId = std::forward<TrafficMirrorTargetIdT>(value);
  }
  template <typename TrafficMirrorTargetIdT = Aws::String>
  CreateTrafficMirrorSessionRequest& WithTrafficMirrorTargetId(TrafficMirrorTargetIdT&& value) {
    SetTrafficMirrorTargetId(std::forward<TrafficMirrorTargetIdT>(value));
    return *this;
  }
  ///@}

  ///@{
  /**
   * <p>The ID of the Traffic Mirror filter.</p>
   */
  inline const Aws::String& GetTrafficMirrorFilterId() const { return m_trafficMirrorFilterId; }
  inline bool TrafficMirrorFilterIdHasBeenSet() const { return m_trafficMirrorFilterIdHasBeenSet; }
  template <typename TrafficMirrorFilterIdT = Aws::String>
  void SetTrafficMirrorFilterId(TrafficMirrorFilterIdT&& value) {
    m_trafficMirrorFilterIdHasBeenSet = true;
    m_trafficMirrorFilterId = std::forward<TrafficMirrorFilterIdT>(value);
  }
  template <typename TrafficMirrorFilterIdT = Aws::String>
  CreateTrafficMirrorSessionRequest& WithTrafficMirrorFilterId(TrafficMirrorFilterIdT&& value) {
    SetTrafficMirrorFilterId(std::forward<TrafficMirrorFilterIdT>(value));
    return *this;
  }
  ///@}

  ///@{
  /**
   * <p>The number of bytes in each packet to mirror. These are bytes after the VXLAN
   * header. Do not specify this parameter when you want to mirror the entire packet.
   * To mirror a subset of the packet, set this to the length (in bytes) that you
   * want to mirror. For example, if you set this value to 100, then the first 100
   * bytes that meet the filter criteria are copied to the target.</p> <p>If you do
   * not want to mirror the entire packet, use the <code>PacketLength</code>
   * parameter to specify the number of bytes in each packet to mirror.</p> <p>For
   * sessions with Network Load Balancer (NLB) Traffic Mirror targets the default
   * <code>PacketLength</code> will be set to 8500. Valid values are 1-8500. Setting
   * a <code>PacketLength</code> greater than 8500 will result in an error
   * response.</p>
   */
  inline int GetPacketLength() const { return m_packetLength; }
  inline bool PacketLengthHasBeenSet() const { return m_packetLengthHasBeenSet; }
  inline void SetPacketLength(int value) {
    m_packetLengthHasBeenSet = true;
    m_packetLength = value;
  }
  inline CreateTrafficMirrorSessionRequest& WithPacketLength(int value) {
    SetPacketLength(value);
    return *this;
  }
  ///@}

  ///@{
  /**
   * <p>The session number determines the order in which sessions are evaluated when
   * an interface is used by multiple sessions. The first session with a matching
   * filter is the one that mirrors the packets.</p> <p>Valid values are 1-32766.</p>
   */
  inline int GetSessionNumber() const { return m_sessionNumber; }
  inline bool SessionNumberHasBeenSet() const { return m_sessionNumberHasBeenSet; }
  inline void SetSessionNumber(int value) {
    m_sessionNumberHasBeenSet = true;
    m_sessionNumber = value;
  }
  inline CreateTrafficMirrorSessionRequest& WithSessionNumber(int value) {
    SetSessionNumber(value);
    return *this;
  }
  ///@}

  ///@{
  /**
   * <p>The VXLAN ID for the Traffic Mirror session. For more information about the
   * VXLAN protocol, see <a href="https://datatracker.ietf.org/doc/html/rfc7348">RFC
   * 7348</a>. If you do not specify a <code>VirtualNetworkId</code>, an account-wide
   * unique ID is chosen at random.</p>
   */
  inline int GetVirtualNetworkId() const { return m_virtualNetworkId; }
  inline bool VirtualNetworkIdHasBeenSet() const { return m_virtualNetworkIdHasBeenSet; }
  inline void SetVirtualNetworkId(int value) {
    m_virtualNetworkIdHasBeenSet = true;
    m_virtualNetworkId = value;
  }
  inline CreateTrafficMirrorSessionRequest& WithVirtualNetworkId(int value) {
    SetVirtualNetworkId(value);
    return *this;
  }
  ///@}

  ///@{
  /**
   * <p>The description of the Traffic Mirror session.</p>
   */
  inline const Aws::String& GetDescription() const { return m_description; }
  inline bool DescriptionHasBeenSet() const { return m_descriptionHasBeenSet; }
  template <typename DescriptionT = Aws::String>
  void SetDescription(DescriptionT&& value) {
    m_descriptionHasBeenSet = true;
    m_description = std::forward<DescriptionT>(value);
  }
  template <typename DescriptionT = Aws::String>
  CreateTrafficMirrorSessionRequest& WithDescription(DescriptionT&& value) {
    SetDescription(std::forward<DescriptionT>(value));
    return *this;
  }
  ///@}

  ///@{
  /**
   * <p>The tags to assign to a Traffic Mirror session.</p>
   */
  inline const Aws::Vector<TagSpecification>& GetTagSpecifications() const { return m_tagSpecifications; }
  inline bool TagSpecificationsHasBeenSet() const { return m_tagSpecificationsHasBeenSet; }
  template <typename TagSpecificationsT = Aws::Vector<TagSpecification>>
  void SetTagSpecifications(TagSpecificationsT&& value) {
    m_tagSpecificationsHasBeenSet = true;
    m_tagSpecifications = std::forward<TagSpecificationsT>(value);
  }
  template <typename TagSpecificationsT = Aws::Vector<TagSpecification>>
  CreateTrafficMirrorSessionRequest& WithTagSpecifications(TagSpecificationsT&& value) {
    SetTagSpecifications(std::forward<TagSpecificationsT>(value));
    return *this;
  }
  template <typename TagSpecificationsT = TagSpecification>
  CreateTrafficMirrorSessionRequest& AddTagSpecifications(TagSpecificationsT&& value) {
    m_tagSpecificationsHasBeenSet = true;
    m_tagSpecifications.emplace_back(std::forward<TagSpecificationsT>(value));
    return *this;
  }
  ///@}

  ///@{
  /**
   * <p>Checks whether you have the required permissions for the action, without
   * actually making the request, and provides an error response. If you have the
   * required permissions, the error response is <code>DryRunOperation</code>.
   * Otherwise, it is <code>UnauthorizedOperation</code>.</p>
   */
  inline bool GetDryRun() const { return m_dryRun; }
  inline bool DryRunHasBeenSet() const { return m_dryRunHasBeenSet; }
  inline void SetDryRun(bool value) {
    m_dryRunHasBeenSet = true;
    m_dryRun = value;
  }
  inline CreateTrafficMirrorSessionRequest& WithDryRun(bool value) {
    SetDryRun(value);
    return *this;
  }
  ///@}

  ///@{
  /**
   * <p>Unique, case-sensitive identifier that you provide to ensure the idempotency
   * of the request. For more information, see <a
   * href="https://docs.aws.amazon.com/ec2/latest/devguide/ec2-api-idempotency.html">How
   * to ensure idempotency</a>.</p>
   */
  inline const Aws::String& GetClientToken() const { return m_clientToken; }
  inline bool ClientTokenHasBeenSet() const { return m_clientTokenHasBeenSet; }
  template <typename ClientTokenT = Aws::String>
  void SetClientToken(ClientTokenT&& value) {
    m_clientTokenHasBeenSet = true;
    m_clientToken = std::forward<ClientTokenT>(value);
  }
  template <typename ClientTokenT = Aws::String>
  CreateTrafficMirrorSessionRequest& WithClientToken(ClientTokenT&& value) {
    SetClientToken(std::forward<ClientTokenT>(value));
    return *this;
  }
  ///@}
 private:
  Aws::String m_networkInterfaceId;

  Aws::String m_trafficMirrorTargetId;

  Aws::String m_trafficMirrorFilterId;

  int m_packetLength{0};

  int m_sessionNumber{0};

  int m_virtualNetworkId{0};

  Aws::String m_description;

  Aws::Vector<TagSpecification> m_tagSpecifications;

  bool m_dryRun{false};

  Aws::String m_clientToken{Aws::Utils::UUID::PseudoRandomUUID()};
  bool m_networkInterfaceIdHasBeenSet = false;
  bool m_trafficMirrorTargetIdHasBeenSet = false;
  bool m_trafficMirrorFilterIdHasBeenSet = false;
  bool m_packetLengthHasBeenSet = false;
  bool m_sessionNumberHasBeenSet = false;
  bool m_virtualNetworkIdHasBeenSet = false;
  bool m_descriptionHasBeenSet = false;
  bool m_tagSpecificationsHasBeenSet = false;
  bool m_dryRunHasBeenSet = false;
  bool m_clientTokenHasBeenSet = true;
};

}  // namespace Model
}  // namespace EC2
}  // namespace Aws
