/**
 * Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
 * SPDX-License-Identifier: Apache-2.0.
 */

#pragma once
#include <aws/core/utils/memory/stl/AWSStreamFwd.h>
#include <aws/core/utils/memory/stl/AWSString.h>
#include <aws/ec2/EC2_EXPORTS.h>
#include <aws/ec2/model/VpnState.h>
#include <aws/ec2/model/VpnStaticRouteSource.h>

#include <utility>

namespace Aws {
namespace Utils {
namespace Xml {
class XmlNode;
}  // namespace Xml
}  // namespace Utils
namespace EC2 {
namespace Model {

/**
 * <p>Describes a static route for a VPN connection.</p><p><h3>See Also:</h3>   <a
 * href="http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/VpnStaticRoute">AWS
 * API Reference</a></p>
 */
class VpnStaticRoute {
 public:
  AWS_EC2_API VpnStaticRoute() = default;
  AWS_EC2_API VpnStaticRoute(const Aws::Utils::Xml::XmlNode& xmlNode);
  AWS_EC2_API VpnStaticRoute& operator=(const Aws::Utils::Xml::XmlNode& xmlNode);

  AWS_EC2_API void OutputToStream(Aws::OStream& ostream, const char* location, unsigned index, const char* locationValue) const;
  AWS_EC2_API void OutputToStream(Aws::OStream& oStream, const char* location) const;

  ///@{
  /**
   * <p>The CIDR block associated with the local subnet of the customer data
   * center.</p>
   */
  inline const Aws::String& GetDestinationCidrBlock() const { return m_destinationCidrBlock; }
  inline bool DestinationCidrBlockHasBeenSet() const { return m_destinationCidrBlockHasBeenSet; }
  template <typename DestinationCidrBlockT = Aws::String>
  void SetDestinationCidrBlock(DestinationCidrBlockT&& value) {
    m_destinationCidrBlockHasBeenSet = true;
    m_destinationCidrBlock = std::forward<DestinationCidrBlockT>(value);
  }
  template <typename DestinationCidrBlockT = Aws::String>
  VpnStaticRoute& WithDestinationCidrBlock(DestinationCidrBlockT&& value) {
    SetDestinationCidrBlock(std::forward<DestinationCidrBlockT>(value));
    return *this;
  }
  ///@}

  ///@{
  /**
   * <p>Indicates how the routes were provided.</p>
   */
  inline VpnStaticRouteSource GetSource() const { return m_source; }
  inline bool SourceHasBeenSet() const { return m_sourceHasBeenSet; }
  inline void SetSource(VpnStaticRouteSource value) {
    m_sourceHasBeenSet = true;
    m_source = value;
  }
  inline VpnStaticRoute& WithSource(VpnStaticRouteSource value) {
    SetSource(value);
    return *this;
  }
  ///@}

  ///@{
  /**
   * <p>The current state of the static route.</p>
   */
  inline VpnState GetState() const { return m_state; }
  inline bool StateHasBeenSet() const { return m_stateHasBeenSet; }
  inline void SetState(VpnState value) {
    m_stateHasBeenSet = true;
    m_state = value;
  }
  inline VpnStaticRoute& WithState(VpnState value) {
    SetState(value);
    return *this;
  }
  ///@}
 private:
  Aws::String m_destinationCidrBlock;

  VpnStaticRouteSource m_source{VpnStaticRouteSource::NOT_SET};

  VpnState m_state{VpnState::NOT_SET};
  bool m_destinationCidrBlockHasBeenSet = false;
  bool m_sourceHasBeenSet = false;
  bool m_stateHasBeenSet = false;
};

}  // namespace Model
}  // namespace EC2
}  // namespace Aws
