/**
 * Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
 * SPDX-License-Identifier: Apache-2.0.
 */

#pragma once
#include <aws/core/utils/DateTime.h>
#include <aws/core/utils/memory/stl/AWSString.h>
#include <aws/core/utils/memory/stl/AWSVector.h>
#include <aws/ec2/EC2Request.h>
#include <aws/ec2/EC2_EXPORTS.h>
#include <aws/ec2/model/Filter.h>
#include <aws/ec2/model/InstanceType.h>

#include <utility>

namespace Aws {
namespace EC2 {
namespace Model {

/**
 * <p>Contains the parameters for DescribeSpotPriceHistory.</p><p><h3>See
 * Also:</h3>   <a
 * href="http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DescribeSpotPriceHistoryRequest">AWS
 * API Reference</a></p>
 */
class DescribeSpotPriceHistoryRequest : public EC2Request {
 public:
  AWS_EC2_API DescribeSpotPriceHistoryRequest() = default;

  // Service request name is the Operation name which will send this request out,
  // each operation should has unique request name, so that we can get operation's name from this request.
  // Note: this is not true for response, multiple operations may have the same response name,
  // so we can not get operation's name from response.
  inline virtual const char* GetServiceRequestName() const override { return "DescribeSpotPriceHistory"; }

  AWS_EC2_API Aws::String SerializePayload() const override;

 protected:
  AWS_EC2_API void DumpBodyToUrl(Aws::Http::URI& uri) const override;

 public:
  ///@{
  /**
   * <p>Filters the results by the specified ID of the Availability Zone.</p>
   * <p>Either <code>AvailabilityZone</code> or <code>AvailabilityZoneId</code> can
   * be specified, but not both</p>
   */
  inline const Aws::String& GetAvailabilityZoneId() const { return m_availabilityZoneId; }
  inline bool AvailabilityZoneIdHasBeenSet() const { return m_availabilityZoneIdHasBeenSet; }
  template <typename AvailabilityZoneIdT = Aws::String>
  void SetAvailabilityZoneId(AvailabilityZoneIdT&& value) {
    m_availabilityZoneIdHasBeenSet = true;
    m_availabilityZoneId = std::forward<AvailabilityZoneIdT>(value);
  }
  template <typename AvailabilityZoneIdT = Aws::String>
  DescribeSpotPriceHistoryRequest& WithAvailabilityZoneId(AvailabilityZoneIdT&& value) {
    SetAvailabilityZoneId(std::forward<AvailabilityZoneIdT>(value));
    return *this;
  }
  ///@}

  ///@{
  /**
   * <p>Checks whether you have the required permissions for the action, without
   * actually making the request, and provides an error response. If you have the
   * required permissions, the error response is <code>DryRunOperation</code>.
   * Otherwise, it is <code>UnauthorizedOperation</code>.</p>
   */
  inline bool GetDryRun() const { return m_dryRun; }
  inline bool DryRunHasBeenSet() const { return m_dryRunHasBeenSet; }
  inline void SetDryRun(bool value) {
    m_dryRunHasBeenSet = true;
    m_dryRun = value;
  }
  inline DescribeSpotPriceHistoryRequest& WithDryRun(bool value) {
    SetDryRun(value);
    return *this;
  }
  ///@}

  ///@{
  /**
   * <p>The date and time, up to the past 90 days, from which to start retrieving the
   * price history data, in UTC format (for example,
   * <i>YYYY</i>-<i>MM</i>-<i>DD</i>T<i>HH</i>:<i>MM</i>:<i>SS</i>Z).</p>
   */
  inline const Aws::Utils::DateTime& GetStartTime() const { return m_startTime; }
  inline bool StartTimeHasBeenSet() const { return m_startTimeHasBeenSet; }
  template <typename StartTimeT = Aws::Utils::DateTime>
  void SetStartTime(StartTimeT&& value) {
    m_startTimeHasBeenSet = true;
    m_startTime = std::forward<StartTimeT>(value);
  }
  template <typename StartTimeT = Aws::Utils::DateTime>
  DescribeSpotPriceHistoryRequest& WithStartTime(StartTimeT&& value) {
    SetStartTime(std::forward<StartTimeT>(value));
    return *this;
  }
  ///@}

  ///@{
  /**
   * <p>The date and time, up to the current date, from which to stop retrieving the
   * price history data, in UTC format (for example,
   * <i>YYYY</i>-<i>MM</i>-<i>DD</i>T<i>HH</i>:<i>MM</i>:<i>SS</i>Z).</p>
   */
  inline const Aws::Utils::DateTime& GetEndTime() const { return m_endTime; }
  inline bool EndTimeHasBeenSet() const { return m_endTimeHasBeenSet; }
  template <typename EndTimeT = Aws::Utils::DateTime>
  void SetEndTime(EndTimeT&& value) {
    m_endTimeHasBeenSet = true;
    m_endTime = std::forward<EndTimeT>(value);
  }
  template <typename EndTimeT = Aws::Utils::DateTime>
  DescribeSpotPriceHistoryRequest& WithEndTime(EndTimeT&& value) {
    SetEndTime(std::forward<EndTimeT>(value));
    return *this;
  }
  ///@}

  ///@{
  /**
   * <p>Filters the results by the specified instance types.</p>
   */
  inline const Aws::Vector<InstanceType>& GetInstanceTypes() const { return m_instanceTypes; }
  inline bool InstanceTypesHasBeenSet() const { return m_instanceTypesHasBeenSet; }
  template <typename InstanceTypesT = Aws::Vector<InstanceType>>
  void SetInstanceTypes(InstanceTypesT&& value) {
    m_instanceTypesHasBeenSet = true;
    m_instanceTypes = std::forward<InstanceTypesT>(value);
  }
  template <typename InstanceTypesT = Aws::Vector<InstanceType>>
  DescribeSpotPriceHistoryRequest& WithInstanceTypes(InstanceTypesT&& value) {
    SetInstanceTypes(std::forward<InstanceTypesT>(value));
    return *this;
  }
  inline DescribeSpotPriceHistoryRequest& AddInstanceTypes(InstanceType value) {
    m_instanceTypesHasBeenSet = true;
    m_instanceTypes.push_back(value);
    return *this;
  }
  ///@}

  ///@{
  /**
   * <p>Filters the results by the specified basic product descriptions.</p>
   */
  inline const Aws::Vector<Aws::String>& GetProductDescriptions() const { return m_productDescriptions; }
  inline bool ProductDescriptionsHasBeenSet() const { return m_productDescriptionsHasBeenSet; }
  template <typename ProductDescriptionsT = Aws::Vector<Aws::String>>
  void SetProductDescriptions(ProductDescriptionsT&& value) {
    m_productDescriptionsHasBeenSet = true;
    m_productDescriptions = std::forward<ProductDescriptionsT>(value);
  }
  template <typename ProductDescriptionsT = Aws::Vector<Aws::String>>
  DescribeSpotPriceHistoryRequest& WithProductDescriptions(ProductDescriptionsT&& value) {
    SetProductDescriptions(std::forward<ProductDescriptionsT>(value));
    return *this;
  }
  template <typename ProductDescriptionsT = Aws::String>
  DescribeSpotPriceHistoryRequest& AddProductDescriptions(ProductDescriptionsT&& value) {
    m_productDescriptionsHasBeenSet = true;
    m_productDescriptions.emplace_back(std::forward<ProductDescriptionsT>(value));
    return *this;
  }
  ///@}

  ///@{
  /**
   * <p>The filters.</p> <ul> <li> <p> <code>availability-zone</code> - The
   * Availability Zone for which prices should be returned.</p> </li> <li> <p>
   * <code>availability-zone-id</code> - The ID of the Availability Zone for which
   * prices should be returned.</p> </li> <li> <p> <code>instance-type</code> - The
   * type of instance (for example, <code>m3.medium</code>).</p> </li> <li> <p>
   * <code>product-description</code> - The product description for the Spot price
   * (<code>Linux/UNIX</code> | <code>Red Hat Enterprise Linux</code> | <code>SUSE
   * Linux</code> | <code>Windows</code> | <code>Linux/UNIX (Amazon VPC)</code> |
   * <code>Red Hat Enterprise Linux (Amazon VPC)</code> | <code>SUSE Linux (Amazon
   * VPC)</code> | <code>Windows (Amazon VPC)</code>).</p> </li> <li> <p>
   * <code>spot-price</code> - The Spot price. The value must match exactly (or use
   * wildcards; greater than or less than comparison is not supported).</p> </li>
   * <li> <p> <code>timestamp</code> - The time stamp of the Spot price history, in
   * UTC format (for example, <i>ddd MMM dd HH</i>:<i>mm</i>:<i>ss</i> UTC
   * <i>YYYY</i>). You can use wildcards (<code>*</code> and <code>?</code>). Greater
   * than or less than comparison is not supported.</p> </li> </ul>
   */
  inline const Aws::Vector<Filter>& GetFilters() const { return m_filters; }
  inline bool FiltersHasBeenSet() const { return m_filtersHasBeenSet; }
  template <typename FiltersT = Aws::Vector<Filter>>
  void SetFilters(FiltersT&& value) {
    m_filtersHasBeenSet = true;
    m_filters = std::forward<FiltersT>(value);
  }
  template <typename FiltersT = Aws::Vector<Filter>>
  DescribeSpotPriceHistoryRequest& WithFilters(FiltersT&& value) {
    SetFilters(std::forward<FiltersT>(value));
    return *this;
  }
  template <typename FiltersT = Filter>
  DescribeSpotPriceHistoryRequest& AddFilters(FiltersT&& value) {
    m_filtersHasBeenSet = true;
    m_filters.emplace_back(std::forward<FiltersT>(value));
    return *this;
  }
  ///@}

  ///@{
  /**
   * <p>Filters the results by the specified Availability Zone.</p> <p>Either
   * <code>AvailabilityZone</code> or <code>AvailabilityZoneId</code> can be
   * specified, but not both</p>
   */
  inline const Aws::String& GetAvailabilityZone() const { return m_availabilityZone; }
  inline bool AvailabilityZoneHasBeenSet() const { return m_availabilityZoneHasBeenSet; }
  template <typename AvailabilityZoneT = Aws::String>
  void SetAvailabilityZone(AvailabilityZoneT&& value) {
    m_availabilityZoneHasBeenSet = true;
    m_availabilityZone = std::forward<AvailabilityZoneT>(value);
  }
  template <typename AvailabilityZoneT = Aws::String>
  DescribeSpotPriceHistoryRequest& WithAvailabilityZone(AvailabilityZoneT&& value) {
    SetAvailabilityZone(std::forward<AvailabilityZoneT>(value));
    return *this;
  }
  ///@}

  ///@{
  /**
   * <p>The maximum number of items to return for this request. To get the next page
   * of items, make another request with the token returned in the output. For more
   * information, see <a
   * href="https://docs.aws.amazon.com/AWSEC2/latest/APIReference/Query-Requests.html#api-pagination">Pagination</a>.</p>
   */
  inline int GetMaxResults() const { return m_maxResults; }
  inline bool MaxResultsHasBeenSet() const { return m_maxResultsHasBeenSet; }
  inline void SetMaxResults(int value) {
    m_maxResultsHasBeenSet = true;
    m_maxResults = value;
  }
  inline DescribeSpotPriceHistoryRequest& WithMaxResults(int value) {
    SetMaxResults(value);
    return *this;
  }
  ///@}

  ///@{
  /**
   * <p>The token returned from a previous paginated request. Pagination continues
   * from the end of the items returned by the previous request.</p>
   */
  inline const Aws::String& GetNextToken() const { return m_nextToken; }
  inline bool NextTokenHasBeenSet() const { return m_nextTokenHasBeenSet; }
  template <typename NextTokenT = Aws::String>
  void SetNextToken(NextTokenT&& value) {
    m_nextTokenHasBeenSet = true;
    m_nextToken = std::forward<NextTokenT>(value);
  }
  template <typename NextTokenT = Aws::String>
  DescribeSpotPriceHistoryRequest& WithNextToken(NextTokenT&& value) {
    SetNextToken(std::forward<NextTokenT>(value));
    return *this;
  }
  ///@}
 private:
  Aws::String m_availabilityZoneId;

  bool m_dryRun{false};

  Aws::Utils::DateTime m_startTime{};

  Aws::Utils::DateTime m_endTime{};

  Aws::Vector<InstanceType> m_instanceTypes;

  Aws::Vector<Aws::String> m_productDescriptions;

  Aws::Vector<Filter> m_filters;

  Aws::String m_availabilityZone;

  int m_maxResults{0};

  Aws::String m_nextToken;
  bool m_availabilityZoneIdHasBeenSet = false;
  bool m_dryRunHasBeenSet = false;
  bool m_startTimeHasBeenSet = false;
  bool m_endTimeHasBeenSet = false;
  bool m_instanceTypesHasBeenSet = false;
  bool m_productDescriptionsHasBeenSet = false;
  bool m_filtersHasBeenSet = false;
  bool m_availabilityZoneHasBeenSet = false;
  bool m_maxResultsHasBeenSet = false;
  bool m_nextTokenHasBeenSet = false;
};

}  // namespace Model
}  // namespace EC2
}  // namespace Aws
