/**
 * Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
 * SPDX-License-Identifier: Apache-2.0.
 */

#pragma once
#include <aws/core/utils/memory/stl/AWSString.h>
#include <aws/core/utils/memory/stl/AWSVector.h>
#include <aws/ec2/EC2Request.h>
#include <aws/ec2/EC2_EXPORTS.h>
#include <aws/ec2/model/Filter.h>

#include <utility>

namespace Aws {
namespace EC2 {
namespace Model {

/**
 */
class DescribeMacModificationTasksRequest : public EC2Request {
 public:
  AWS_EC2_API DescribeMacModificationTasksRequest() = default;

  // Service request name is the Operation name which will send this request out,
  // each operation should has unique request name, so that we can get operation's name from this request.
  // Note: this is not true for response, multiple operations may have the same response name,
  // so we can not get operation's name from response.
  inline virtual const char* GetServiceRequestName() const override { return "DescribeMacModificationTasks"; }

  AWS_EC2_API Aws::String SerializePayload() const override;

 protected:
  AWS_EC2_API void DumpBodyToUrl(Aws::Http::URI& uri) const override;

 public:
  ///@{
  /**
   * <p>Checks whether you have the required permissions for the action, without
   * actually making the request, and provides an error response. If you have the
   * required permissions, the error response is <code>DryRunOperation</code>.
   * Otherwise, it is <code>UnauthorizedOperation</code>.</p>
   */
  inline bool GetDryRun() const { return m_dryRun; }
  inline bool DryRunHasBeenSet() const { return m_dryRunHasBeenSet; }
  inline void SetDryRun(bool value) {
    m_dryRunHasBeenSet = true;
    m_dryRun = value;
  }
  inline DescribeMacModificationTasksRequest& WithDryRun(bool value) {
    SetDryRun(value);
    return *this;
  }
  ///@}

  ///@{
  /**
   * <p>Specifies one or more filters for the request:</p> <ul> <li> <p>
   * <code>instance-id</code> - The ID of the instance for which the task was
   * created.</p> </li> <li> <p> <code>task-state</code> - The state of the task
   * (<code>successful</code> | <code>failed</code> | <code>in-progress</code> |
   * <code>pending</code>).</p> </li> <li> <p>
   * <code>mac-system-integrity-protection-configuration.sip-status</code> - The
   * overall SIP state requested in the task (<code>enabled</code> |
   * <code>disabled</code>).</p> </li> <li> <p> <code>start-time</code> - The date
   * and time the task was created.</p> </li> <li> <p> <code>task-type</code> - The
   * type of task (<code>sip-modification</code> |
   * <code>volume-ownership-delegation</code>).</p> </li> </ul>
   */
  inline const Aws::Vector<Filter>& GetFilters() const { return m_filters; }
  inline bool FiltersHasBeenSet() const { return m_filtersHasBeenSet; }
  template <typename FiltersT = Aws::Vector<Filter>>
  void SetFilters(FiltersT&& value) {
    m_filtersHasBeenSet = true;
    m_filters = std::forward<FiltersT>(value);
  }
  template <typename FiltersT = Aws::Vector<Filter>>
  DescribeMacModificationTasksRequest& WithFilters(FiltersT&& value) {
    SetFilters(std::forward<FiltersT>(value));
    return *this;
  }
  template <typename FiltersT = Filter>
  DescribeMacModificationTasksRequest& AddFilters(FiltersT&& value) {
    m_filtersHasBeenSet = true;
    m_filters.emplace_back(std::forward<FiltersT>(value));
    return *this;
  }
  ///@}

  ///@{
  /**
   * <p>The ID of task.</p>
   */
  inline const Aws::Vector<Aws::String>& GetMacModificationTaskIds() const { return m_macModificationTaskIds; }
  inline bool MacModificationTaskIdsHasBeenSet() const { return m_macModificationTaskIdsHasBeenSet; }
  template <typename MacModificationTaskIdsT = Aws::Vector<Aws::String>>
  void SetMacModificationTaskIds(MacModificationTaskIdsT&& value) {
    m_macModificationTaskIdsHasBeenSet = true;
    m_macModificationTaskIds = std::forward<MacModificationTaskIdsT>(value);
  }
  template <typename MacModificationTaskIdsT = Aws::Vector<Aws::String>>
  DescribeMacModificationTasksRequest& WithMacModificationTaskIds(MacModificationTaskIdsT&& value) {
    SetMacModificationTaskIds(std::forward<MacModificationTaskIdsT>(value));
    return *this;
  }
  template <typename MacModificationTaskIdsT = Aws::String>
  DescribeMacModificationTasksRequest& AddMacModificationTaskIds(MacModificationTaskIdsT&& value) {
    m_macModificationTaskIdsHasBeenSet = true;
    m_macModificationTaskIds.emplace_back(std::forward<MacModificationTaskIdsT>(value));
    return *this;
  }
  ///@}

  ///@{
  /**
   * <p>The maximum number of results to return for the request in a single page. The
   * remaining results can be seen by sending another request with the returned
   * <code>nextToken</code> value. This value can be between 5 and 500. If
   * <code>maxResults</code> is given a larger value than 500, you receive an
   * error.</p>
   */
  inline int GetMaxResults() const { return m_maxResults; }
  inline bool MaxResultsHasBeenSet() const { return m_maxResultsHasBeenSet; }
  inline void SetMaxResults(int value) {
    m_maxResultsHasBeenSet = true;
    m_maxResults = value;
  }
  inline DescribeMacModificationTasksRequest& WithMaxResults(int value) {
    SetMaxResults(value);
    return *this;
  }
  ///@}

  ///@{
  /**
   * <p>The token to use to retrieve the next page of results.</p>
   */
  inline const Aws::String& GetNextToken() const { return m_nextToken; }
  inline bool NextTokenHasBeenSet() const { return m_nextTokenHasBeenSet; }
  template <typename NextTokenT = Aws::String>
  void SetNextToken(NextTokenT&& value) {
    m_nextTokenHasBeenSet = true;
    m_nextToken = std::forward<NextTokenT>(value);
  }
  template <typename NextTokenT = Aws::String>
  DescribeMacModificationTasksRequest& WithNextToken(NextTokenT&& value) {
    SetNextToken(std::forward<NextTokenT>(value));
    return *this;
  }
  ///@}
 private:
  bool m_dryRun{false};

  Aws::Vector<Filter> m_filters;

  Aws::Vector<Aws::String> m_macModificationTaskIds;

  int m_maxResults{0};

  Aws::String m_nextToken;
  bool m_dryRunHasBeenSet = false;
  bool m_filtersHasBeenSet = false;
  bool m_macModificationTaskIdsHasBeenSet = false;
  bool m_maxResultsHasBeenSet = false;
  bool m_nextTokenHasBeenSet = false;
};

}  // namespace Model
}  // namespace EC2
}  // namespace Aws
