/**
 * Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
 * SPDX-License-Identifier: Apache-2.0.
 */

#pragma once
#include <aws/core/utils/UUID.h>
#include <aws/core/utils/memory/stl/AWSString.h>
#include <aws/core/utils/memory/stl/AWSVector.h>
#include <aws/ec2/EC2Request.h>
#include <aws/ec2/EC2_EXPORTS.h>
#include <aws/ec2/model/AddPrefixListEntry.h>
#include <aws/ec2/model/TagSpecification.h>

#include <utility>

namespace Aws {
namespace EC2 {
namespace Model {

/**
 */
class CreateManagedPrefixListRequest : public EC2Request {
 public:
  AWS_EC2_API CreateManagedPrefixListRequest() = default;

  // Service request name is the Operation name which will send this request out,
  // each operation should has unique request name, so that we can get operation's name from this request.
  // Note: this is not true for response, multiple operations may have the same response name,
  // so we can not get operation's name from response.
  inline virtual const char* GetServiceRequestName() const override { return "CreateManagedPrefixList"; }

  AWS_EC2_API Aws::String SerializePayload() const override;

 protected:
  AWS_EC2_API void DumpBodyToUrl(Aws::Http::URI& uri) const override;

 public:
  ///@{
  /**
   * <p>Checks whether you have the required permissions for the action, without
   * actually making the request, and provides an error response. If you have the
   * required permissions, the error response is <code>DryRunOperation</code>.
   * Otherwise, it is <code>UnauthorizedOperation</code>.</p>
   */
  inline bool GetDryRun() const { return m_dryRun; }
  inline bool DryRunHasBeenSet() const { return m_dryRunHasBeenSet; }
  inline void SetDryRun(bool value) {
    m_dryRunHasBeenSet = true;
    m_dryRun = value;
  }
  inline CreateManagedPrefixListRequest& WithDryRun(bool value) {
    SetDryRun(value);
    return *this;
  }
  ///@}

  ///@{
  /**
   * <p>A name for the prefix list.</p> <p>Constraints: Up to 255 characters in
   * length. The name cannot start with <code>com.amazonaws</code>.</p>
   */
  inline const Aws::String& GetPrefixListName() const { return m_prefixListName; }
  inline bool PrefixListNameHasBeenSet() const { return m_prefixListNameHasBeenSet; }
  template <typename PrefixListNameT = Aws::String>
  void SetPrefixListName(PrefixListNameT&& value) {
    m_prefixListNameHasBeenSet = true;
    m_prefixListName = std::forward<PrefixListNameT>(value);
  }
  template <typename PrefixListNameT = Aws::String>
  CreateManagedPrefixListRequest& WithPrefixListName(PrefixListNameT&& value) {
    SetPrefixListName(std::forward<PrefixListNameT>(value));
    return *this;
  }
  ///@}

  ///@{
  /**
   * <p>One or more entries for the prefix list.</p>
   */
  inline const Aws::Vector<AddPrefixListEntry>& GetEntries() const { return m_entries; }
  inline bool EntriesHasBeenSet() const { return m_entriesHasBeenSet; }
  template <typename EntriesT = Aws::Vector<AddPrefixListEntry>>
  void SetEntries(EntriesT&& value) {
    m_entriesHasBeenSet = true;
    m_entries = std::forward<EntriesT>(value);
  }
  template <typename EntriesT = Aws::Vector<AddPrefixListEntry>>
  CreateManagedPrefixListRequest& WithEntries(EntriesT&& value) {
    SetEntries(std::forward<EntriesT>(value));
    return *this;
  }
  template <typename EntriesT = AddPrefixListEntry>
  CreateManagedPrefixListRequest& AddEntries(EntriesT&& value) {
    m_entriesHasBeenSet = true;
    m_entries.emplace_back(std::forward<EntriesT>(value));
    return *this;
  }
  ///@}

  ///@{
  /**
   * <p>The maximum number of entries for the prefix list.</p>
   */
  inline int GetMaxEntries() const { return m_maxEntries; }
  inline bool MaxEntriesHasBeenSet() const { return m_maxEntriesHasBeenSet; }
  inline void SetMaxEntries(int value) {
    m_maxEntriesHasBeenSet = true;
    m_maxEntries = value;
  }
  inline CreateManagedPrefixListRequest& WithMaxEntries(int value) {
    SetMaxEntries(value);
    return *this;
  }
  ///@}

  ///@{
  /**
   * <p>The tags to apply to the prefix list during creation.</p>
   */
  inline const Aws::Vector<TagSpecification>& GetTagSpecifications() const { return m_tagSpecifications; }
  inline bool TagSpecificationsHasBeenSet() const { return m_tagSpecificationsHasBeenSet; }
  template <typename TagSpecificationsT = Aws::Vector<TagSpecification>>
  void SetTagSpecifications(TagSpecificationsT&& value) {
    m_tagSpecificationsHasBeenSet = true;
    m_tagSpecifications = std::forward<TagSpecificationsT>(value);
  }
  template <typename TagSpecificationsT = Aws::Vector<TagSpecification>>
  CreateManagedPrefixListRequest& WithTagSpecifications(TagSpecificationsT&& value) {
    SetTagSpecifications(std::forward<TagSpecificationsT>(value));
    return *this;
  }
  template <typename TagSpecificationsT = TagSpecification>
  CreateManagedPrefixListRequest& AddTagSpecifications(TagSpecificationsT&& value) {
    m_tagSpecificationsHasBeenSet = true;
    m_tagSpecifications.emplace_back(std::forward<TagSpecificationsT>(value));
    return *this;
  }
  ///@}

  ///@{
  /**
   * <p>The IP address type.</p> <p>Valid Values: <code>IPv4</code> |
   * <code>IPv6</code> </p>
   */
  inline const Aws::String& GetAddressFamily() const { return m_addressFamily; }
  inline bool AddressFamilyHasBeenSet() const { return m_addressFamilyHasBeenSet; }
  template <typename AddressFamilyT = Aws::String>
  void SetAddressFamily(AddressFamilyT&& value) {
    m_addressFamilyHasBeenSet = true;
    m_addressFamily = std::forward<AddressFamilyT>(value);
  }
  template <typename AddressFamilyT = Aws::String>
  CreateManagedPrefixListRequest& WithAddressFamily(AddressFamilyT&& value) {
    SetAddressFamily(std::forward<AddressFamilyT>(value));
    return *this;
  }
  ///@}

  ///@{
  /**
   * <p>Unique, case-sensitive identifier you provide to ensure the idempotency of
   * the request. For more information, see <a
   * href="https://docs.aws.amazon.com/ec2/latest/devguide/ec2-api-idempotency.html">Ensuring
   * idempotency</a>.</p> <p>Constraints: Up to 255 UTF-8 characters in length.</p>
   */
  inline const Aws::String& GetClientToken() const { return m_clientToken; }
  inline bool ClientTokenHasBeenSet() const { return m_clientTokenHasBeenSet; }
  template <typename ClientTokenT = Aws::String>
  void SetClientToken(ClientTokenT&& value) {
    m_clientTokenHasBeenSet = true;
    m_clientToken = std::forward<ClientTokenT>(value);
  }
  template <typename ClientTokenT = Aws::String>
  CreateManagedPrefixListRequest& WithClientToken(ClientTokenT&& value) {
    SetClientToken(std::forward<ClientTokenT>(value));
    return *this;
  }
  ///@}
 private:
  bool m_dryRun{false};

  Aws::String m_prefixListName;

  Aws::Vector<AddPrefixListEntry> m_entries;

  int m_maxEntries{0};

  Aws::Vector<TagSpecification> m_tagSpecifications;

  Aws::String m_addressFamily;

  Aws::String m_clientToken{Aws::Utils::UUID::PseudoRandomUUID()};
  bool m_dryRunHasBeenSet = false;
  bool m_prefixListNameHasBeenSet = false;
  bool m_entriesHasBeenSet = false;
  bool m_maxEntriesHasBeenSet = false;
  bool m_tagSpecificationsHasBeenSet = false;
  bool m_addressFamilyHasBeenSet = false;
  bool m_clientTokenHasBeenSet = true;
};

}  // namespace Model
}  // namespace EC2
}  // namespace Aws
