/**
 * Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
 * SPDX-License-Identifier: Apache-2.0.
 */

#pragma once
#include <aws/core/utils/memory/stl/AWSStreamFwd.h>
#include <aws/core/utils/memory/stl/AWSString.h>
#include <aws/core/utils/memory/stl/AWSVector.h>
#include <aws/ec2/EC2_EXPORTS.h>
#include <aws/ec2/model/Tag.h>

#include <utility>

namespace Aws {
namespace Utils {
namespace Xml {
class XmlNode;
}  // namespace Xml
}  // namespace Utils
namespace EC2 {
namespace Model {

/**
 * <p>Describes a customer-owned address pool.</p><p><h3>See Also:</h3>   <a
 * href="http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/CoipPool">AWS API
 * Reference</a></p>
 */
class CoipPool {
 public:
  AWS_EC2_API CoipPool() = default;
  AWS_EC2_API CoipPool(const Aws::Utils::Xml::XmlNode& xmlNode);
  AWS_EC2_API CoipPool& operator=(const Aws::Utils::Xml::XmlNode& xmlNode);

  AWS_EC2_API void OutputToStream(Aws::OStream& ostream, const char* location, unsigned index, const char* locationValue) const;
  AWS_EC2_API void OutputToStream(Aws::OStream& oStream, const char* location) const;

  ///@{
  /**
   * <p>The ID of the address pool.</p>
   */
  inline const Aws::String& GetPoolId() const { return m_poolId; }
  inline bool PoolIdHasBeenSet() const { return m_poolIdHasBeenSet; }
  template <typename PoolIdT = Aws::String>
  void SetPoolId(PoolIdT&& value) {
    m_poolIdHasBeenSet = true;
    m_poolId = std::forward<PoolIdT>(value);
  }
  template <typename PoolIdT = Aws::String>
  CoipPool& WithPoolId(PoolIdT&& value) {
    SetPoolId(std::forward<PoolIdT>(value));
    return *this;
  }
  ///@}

  ///@{
  /**
   * <p>The address ranges of the address pool.</p>
   */
  inline const Aws::Vector<Aws::String>& GetPoolCidrs() const { return m_poolCidrs; }
  inline bool PoolCidrsHasBeenSet() const { return m_poolCidrsHasBeenSet; }
  template <typename PoolCidrsT = Aws::Vector<Aws::String>>
  void SetPoolCidrs(PoolCidrsT&& value) {
    m_poolCidrsHasBeenSet = true;
    m_poolCidrs = std::forward<PoolCidrsT>(value);
  }
  template <typename PoolCidrsT = Aws::Vector<Aws::String>>
  CoipPool& WithPoolCidrs(PoolCidrsT&& value) {
    SetPoolCidrs(std::forward<PoolCidrsT>(value));
    return *this;
  }
  template <typename PoolCidrsT = Aws::String>
  CoipPool& AddPoolCidrs(PoolCidrsT&& value) {
    m_poolCidrsHasBeenSet = true;
    m_poolCidrs.emplace_back(std::forward<PoolCidrsT>(value));
    return *this;
  }
  ///@}

  ///@{
  /**
   * <p>The ID of the local gateway route table.</p>
   */
  inline const Aws::String& GetLocalGatewayRouteTableId() const { return m_localGatewayRouteTableId; }
  inline bool LocalGatewayRouteTableIdHasBeenSet() const { return m_localGatewayRouteTableIdHasBeenSet; }
  template <typename LocalGatewayRouteTableIdT = Aws::String>
  void SetLocalGatewayRouteTableId(LocalGatewayRouteTableIdT&& value) {
    m_localGatewayRouteTableIdHasBeenSet = true;
    m_localGatewayRouteTableId = std::forward<LocalGatewayRouteTableIdT>(value);
  }
  template <typename LocalGatewayRouteTableIdT = Aws::String>
  CoipPool& WithLocalGatewayRouteTableId(LocalGatewayRouteTableIdT&& value) {
    SetLocalGatewayRouteTableId(std::forward<LocalGatewayRouteTableIdT>(value));
    return *this;
  }
  ///@}

  ///@{
  /**
   * <p>The tags.</p>
   */
  inline const Aws::Vector<Tag>& GetTags() const { return m_tags; }
  inline bool TagsHasBeenSet() const { return m_tagsHasBeenSet; }
  template <typename TagsT = Aws::Vector<Tag>>
  void SetTags(TagsT&& value) {
    m_tagsHasBeenSet = true;
    m_tags = std::forward<TagsT>(value);
  }
  template <typename TagsT = Aws::Vector<Tag>>
  CoipPool& WithTags(TagsT&& value) {
    SetTags(std::forward<TagsT>(value));
    return *this;
  }
  template <typename TagsT = Tag>
  CoipPool& AddTags(TagsT&& value) {
    m_tagsHasBeenSet = true;
    m_tags.emplace_back(std::forward<TagsT>(value));
    return *this;
  }
  ///@}

  ///@{
  /**
   * <p>The ARN of the address pool.</p>
   */
  inline const Aws::String& GetPoolArn() const { return m_poolArn; }
  inline bool PoolArnHasBeenSet() const { return m_poolArnHasBeenSet; }
  template <typename PoolArnT = Aws::String>
  void SetPoolArn(PoolArnT&& value) {
    m_poolArnHasBeenSet = true;
    m_poolArn = std::forward<PoolArnT>(value);
  }
  template <typename PoolArnT = Aws::String>
  CoipPool& WithPoolArn(PoolArnT&& value) {
    SetPoolArn(std::forward<PoolArnT>(value));
    return *this;
  }
  ///@}
 private:
  Aws::String m_poolId;

  Aws::Vector<Aws::String> m_poolCidrs;

  Aws::String m_localGatewayRouteTableId;

  Aws::Vector<Tag> m_tags;

  Aws::String m_poolArn;
  bool m_poolIdHasBeenSet = false;
  bool m_poolCidrsHasBeenSet = false;
  bool m_localGatewayRouteTableIdHasBeenSet = false;
  bool m_tagsHasBeenSet = false;
  bool m_poolArnHasBeenSet = false;
};

}  // namespace Model
}  // namespace EC2
}  // namespace Aws
