/**
 * Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
 * SPDX-License-Identifier: Apache-2.0.
 */

#pragma once
#include <aws/core/utils/memory/stl/AWSStreamFwd.h>
#include <aws/core/utils/memory/stl/AWSString.h>
#include <aws/ec2/EC2_EXPORTS.h>
#include <aws/ec2/model/InstanceLifecycle.h>
#include <aws/ec2/model/LaunchTemplateAndOverridesResponse.h>

#include <utility>

namespace Aws {
namespace Utils {
namespace Xml {
class XmlNode;
}  // namespace Xml
}  // namespace Utils
namespace EC2 {
namespace Model {

/**
 * <p>Describes the instances that could not be launched by the
 * fleet.</p><p><h3>See Also:</h3>   <a
 * href="http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DescribeFleetError">AWS
 * API Reference</a></p>
 */
class DescribeFleetError {
 public:
  AWS_EC2_API DescribeFleetError() = default;
  AWS_EC2_API DescribeFleetError(const Aws::Utils::Xml::XmlNode& xmlNode);
  AWS_EC2_API DescribeFleetError& operator=(const Aws::Utils::Xml::XmlNode& xmlNode);

  AWS_EC2_API void OutputToStream(Aws::OStream& ostream, const char* location, unsigned index, const char* locationValue) const;
  AWS_EC2_API void OutputToStream(Aws::OStream& oStream, const char* location) const;

  ///@{
  /**
   * <p>The launch templates and overrides that were used for launching the
   * instances. The values that you specify in the Overrides replace the values in
   * the launch template.</p>
   */
  inline const LaunchTemplateAndOverridesResponse& GetLaunchTemplateAndOverrides() const { return m_launchTemplateAndOverrides; }
  inline bool LaunchTemplateAndOverridesHasBeenSet() const { return m_launchTemplateAndOverridesHasBeenSet; }
  template <typename LaunchTemplateAndOverridesT = LaunchTemplateAndOverridesResponse>
  void SetLaunchTemplateAndOverrides(LaunchTemplateAndOverridesT&& value) {
    m_launchTemplateAndOverridesHasBeenSet = true;
    m_launchTemplateAndOverrides = std::forward<LaunchTemplateAndOverridesT>(value);
  }
  template <typename LaunchTemplateAndOverridesT = LaunchTemplateAndOverridesResponse>
  DescribeFleetError& WithLaunchTemplateAndOverrides(LaunchTemplateAndOverridesT&& value) {
    SetLaunchTemplateAndOverrides(std::forward<LaunchTemplateAndOverridesT>(value));
    return *this;
  }
  ///@}

  ///@{
  /**
   * <p>Indicates if the instance that could not be launched was a Spot, On-Demand,
   * Capacity Block, or Interruptible Capacity Reservation instance.</p>
   */
  inline InstanceLifecycle GetLifecycle() const { return m_lifecycle; }
  inline bool LifecycleHasBeenSet() const { return m_lifecycleHasBeenSet; }
  inline void SetLifecycle(InstanceLifecycle value) {
    m_lifecycleHasBeenSet = true;
    m_lifecycle = value;
  }
  inline DescribeFleetError& WithLifecycle(InstanceLifecycle value) {
    SetLifecycle(value);
    return *this;
  }
  ///@}

  ///@{
  /**
   * <p>The error code that indicates why the instance could not be launched. For
   * more information about error codes, see <a
   * href="https://docs.aws.amazon.com/AWSEC2/latest/APIReference/errors-overview.html.html">Error
   * codes</a>.</p>
   */
  inline const Aws::String& GetErrorCode() const { return m_errorCode; }
  inline bool ErrorCodeHasBeenSet() const { return m_errorCodeHasBeenSet; }
  template <typename ErrorCodeT = Aws::String>
  void SetErrorCode(ErrorCodeT&& value) {
    m_errorCodeHasBeenSet = true;
    m_errorCode = std::forward<ErrorCodeT>(value);
  }
  template <typename ErrorCodeT = Aws::String>
  DescribeFleetError& WithErrorCode(ErrorCodeT&& value) {
    SetErrorCode(std::forward<ErrorCodeT>(value));
    return *this;
  }
  ///@}

  ///@{
  /**
   * <p>The error message that describes why the instance could not be launched. For
   * more information about error messages, see <a
   * href="https://docs.aws.amazon.com/AWSEC2/latest/APIReference/errors-overview.html.html">Error
   * codes</a>.</p>
   */
  inline const Aws::String& GetErrorMessage() const { return m_errorMessage; }
  inline bool ErrorMessageHasBeenSet() const { return m_errorMessageHasBeenSet; }
  template <typename ErrorMessageT = Aws::String>
  void SetErrorMessage(ErrorMessageT&& value) {
    m_errorMessageHasBeenSet = true;
    m_errorMessage = std::forward<ErrorMessageT>(value);
  }
  template <typename ErrorMessageT = Aws::String>
  DescribeFleetError& WithErrorMessage(ErrorMessageT&& value) {
    SetErrorMessage(std::forward<ErrorMessageT>(value));
    return *this;
  }
  ///@}
 private:
  LaunchTemplateAndOverridesResponse m_launchTemplateAndOverrides;

  InstanceLifecycle m_lifecycle{InstanceLifecycle::NOT_SET};

  Aws::String m_errorCode;

  Aws::String m_errorMessage;
  bool m_launchTemplateAndOverridesHasBeenSet = false;
  bool m_lifecycleHasBeenSet = false;
  bool m_errorCodeHasBeenSet = false;
  bool m_errorMessageHasBeenSet = false;
};

}  // namespace Model
}  // namespace EC2
}  // namespace Aws
